@@ -14,6 +14,9 @@ __all__ = [
     "DatasetItem",
     "DatasetItemRetrievalError",
     "Frame",
+    "Keypoint",
+    "KeypointsAnnotation",
+    "KeypointsPrediction",
     "LidarScene",
     "LineAnnotation",
     "LinePrediction",
@@ -51,6 +54,8 @@ from .annotation import (
     BoxAnnotation,
     CategoryAnnotation,
     CuboidAnnotation,
+    Keypoint,
+    KeypointsAnnotation,
     LineAnnotation,
     MultiCategoryAnnotation,
     Point,
@@ -123,6 +128,7 @@ from .prediction import (
     BoxPrediction,
     CategoryPrediction,
     CuboidPrediction,
+    KeypointsPrediction,
     LinePrediction,
     PolygonPrediction,
     SegmentationPrediction,
@@ -809,6 +815,7 @@ class NucleusClient:
         Returns:
             Response payload as JSON dict.
         """
+        print(payload, route)
         if payload is None:
             payload = {}
         if requests_command is requests.get:
