@@ -1,31 +1,38 @@
-from typing import List, Optional, Dict, Union
-from .dataset_item import DatasetItem
-from .scene import LidarScene
+from typing import Any, Dict, List, Optional, Union
+
 from .annotation import (
     BoxAnnotation,
+    CategoryAnnotation,
     CuboidAnnotation,
+    MultiCategoryAnnotation,
     PolygonAnnotation,
     SegmentationAnnotation,
 )
+from .constants import (
+    ANNOTATION_METADATA_SCHEMA_KEY,
+    ANNOTATION_UPDATE_KEY,
+    ANNOTATIONS_KEY,
+    ITEMS_KEY,
+    LABELS_KEY,
+    METADATA_KEY,
+    MODEL_ID_KEY,
+    NAME_KEY,
+    REFERENCE_ID_KEY,
+    SCENES_KEY,
+    SEGMENTATIONS_KEY,
+    TAXONOMY_NAME_KEY,
+    TYPE_KEY,
+    UPDATE_KEY,
+)
+from .dataset_item import DatasetItem
 from .prediction import (
     BoxPrediction,
+    CategoryPrediction,
     CuboidPrediction,
     PolygonPrediction,
     SegmentationPrediction,
 )
-from .constants import (
-    ANNOTATION_UPDATE_KEY,
-    NAME_KEY,
-    METADATA_KEY,
-    REFERENCE_ID_KEY,
-    ANNOTATIONS_KEY,
-    ITEMS_KEY,
-    SCENES_KEY,
-    UPDATE_KEY,
-    MODEL_ID_KEY,
-    ANNOTATION_METADATA_SCHEMA_KEY,
-    SEGMENTATIONS_KEY,
-)
+from .scene import LidarScene, VideoScene
 
 
 def construct_append_payload(
@@ -43,7 +50,8 @@ def construct_append_payload(
 
 
 def construct_append_scenes_payload(
-    scene_list: List[LidarScene], update: Optional[bool] = False
+    scene_list: Union[List[LidarScene], List[VideoScene]],
+    update: Optional[bool] = False,
 ) -> dict:
     scenes = []
     for scene in scene_list:
@@ -57,16 +65,29 @@ def construct_annotation_payload(
             BoxAnnotation,
             PolygonAnnotation,
             CuboidAnnotation,
+            CategoryAnnotation,
+            MultiCategoryAnnotation,
             SegmentationAnnotation,
         ]
     ],
     update: bool,
 ) -> dict:
-    annotations = []
-    for annotation_item in annotation_items:
-        annotations.append(annotation_item.to_payload())
-
-    return {ANNOTATIONS_KEY: annotations, ANNOTATION_UPDATE_KEY: update}
+    annotations = [
+        annotation.to_payload()
+        for annotation in annotation_items
+        if not isinstance(annotation, SegmentationAnnotation)
+    ]
+    segmentations = [
+        annotation.to_payload()
+        for annotation in annotation_items
+        if isinstance(annotation, SegmentationAnnotation)
+    ]
+    payload: Dict[str, Any] = {ANNOTATION_UPDATE_KEY: update}
+    if annotations:
+        payload[ANNOTATIONS_KEY] = annotations
+    if segmentations:
+        payload[SEGMENTATIONS_KEY] = segmentations
+    return payload
 
 
 def construct_segmentation_payload(
@@ -84,7 +105,12 @@ def construct_segmentation_payload(
 
 def construct_box_predictions_payload(
     box_predictions: List[
-        Union[BoxPrediction, PolygonPrediction, CuboidPrediction]
+        Union[
+            BoxPrediction,
+            PolygonPrediction,
+            CuboidPrediction,
+            CategoryPrediction,
+        ]
     ],
     update: bool,
 ) -> dict:
@@ -126,3 +152,14 @@ def construct_model_run_creation_payload(
         METADATA_KEY: metadata if metadata else {},
         ANNOTATION_METADATA_SCHEMA_KEY: annotation_metadata_schema,
     }
+
+
+def construct_taxonomy_payload(
+    taxonomy_name: str, taxonomy_type: str, labels: List[str], update: bool
+) -> dict:
+    return {
+        TAXONOMY_NAME_KEY: taxonomy_name,
+        TYPE_KEY: taxonomy_type,
+        LABELS_KEY: labels,
+        UPDATE_KEY: update,
+    }
